import bpy


from bpy.types import (
    Operator,
)
from bpy.props import (
    IntProperty,
)
def get_constraint(context):
    if context.space_data.context == 'BONE_CONSTRAINT' and context.mode == 'POSE':
        # Check for bone constraints in pose mode
        bone = context.active_pose_bone
        if bone:
            index = bone.Bone_Constraint_list_index
            constraints_list = bone.Bone_Constraint_list
            if 0 <= index < len(constraints_list):
                target_constraint_name = constraints_list[index].name
                for constraint in bone.constraints:
                    if constraint.name == target_constraint_name:
                        return constraint
    else:
        # Check for object constraints
        obj = context.active_object
        if obj:
            index = obj.Object_Constraint_list_index
            constraints_list = obj.Object_Constraint_list
            if 0 <= index < len(constraints_list):
                target_constraint_name = constraints_list[index].name
                for constraint in obj.constraints:
                    if constraint.name == target_constraint_name:
                        return constraint
    return None

class CONSTRAINT_OT_add_target(bpy.types.Operator):
    """Add a target to the constraint"""
    bl_idname = "constraint.add_target"
    bl_label = "Add Target"
    bl_options = {'UNDO', 'INTERNAL'}

    @classmethod
    def poll(cls, context):
        constraint = get_constraint(context)
        return constraint

    def execute(self, context):
        if get_constraint(context):
            get_constraint(context).targets.new()
        return {'FINISHED'}


class CONSTRAINT_OT_remove_target(bpy.types.Operator):
    """Remove the target from the constraint"""
    bl_idname = "constraint.remove_target"
    bl_label = "Remove Target"
    bl_options = {'UNDO', 'INTERNAL'}

    index: bpy.props.IntProperty()

    @classmethod
    def poll(cls, context):
        return get_constraint(context)

    def execute(self, context):
        if get_constraint(context):
            tgts = get_constraint(context).targets
            tgts.remove(tgts[self.index])
        return {'FINISHED'}


class CONSTRAINT_OT_normalize_target_weights(bpy.types.Operator):
    """Normalize weights of all target bones"""
    bl_idname = "constraint.normalize_target_weights"
    bl_label = "Normalize Weights"
    bl_options = {'UNDO', 'INTERNAL'}

    @classmethod
    def poll(cls, context):
        return get_constraint(context)

    def execute(self, context):
        if get_constraint(context):
            tgts = get_constraint(context).targets
            total = sum(t.weight for t in tgts)

            if total > 0:
                for t in tgts:
                    t.weight = t.weight / total

        return {'FINISHED'}


class CONSTRAINT_OT_disable_keep_transform(bpy.types.Operator):
    """Set the influence of this constraint to zero while """ \
        """trying to maintain the object's transformation. Other active """ \
        """constraints can still influence the final transformation"""

    bl_idname = "constraint.disable_keep_transform"
    bl_label = "Disable and Keep Transform"
    bl_options = {'UNDO', 'INTERNAL'}

    @classmethod
    def poll(cls, context):
        return get_constraint(context) and get_constraint(context).influence > 0.0

    def execute(self, context):
        if get_constraint(context):
            get_constraint(context).influence = 0.0
        return {'FINISHED'}
        

